#! /bin/sh
# Copyright © 2011 Aleksey Cherepanov <aleksey.4erepanov@gmail.com>.  See LICENSE.
# Script to show status of certain attack
# Usage: astatus.sh [N]
# Where N is number of attack. If omitted then last attack number used.

# TODO: Show name of pattern, John keys.
# It shows for each type: status (running, stopped, finished), c/s,
# guesses, time it is running, time since last guess, approximate count of tries since
# last guess (time * c/s).

. ./config.sh

# We determine attack number.
n=$1
[ -z $n ] && n=$(cat attacks.count)

# We use logs to take status.
# The only thing we should take from John itself is c/s for running
# Johns.
# TODO: Are there log verbosity? Could it be increased to obtain more
#       info from logs?

# Attack prefix
attack=$ATTACK_PREFIX$n$TYPE_PREFIX

# TODO: Sort table by some field (or even provide ability to choose).

# We will prettify our table using w3m.
# NOTE: lynx does not support tables well. So w3m.
(echo '<html><body><table>'
echo '<tr><th>Type </th><th> Status </th><th> C/S </th><th> Guesses
</th><th> Time </th><th> Time since last crack </th><th> Tries since
last crack (time * c/s) </th></tr>'

# We traverse type.
while read type file; do
    session=$attack$type
    # We check status.
    # finished - no .rec file.
    # stopped - .rec file but no john process.
    # running - .rec file and john process.
    if ps -A -o cmd | grep "$JOHN -session:$session" | grep -v grep > /dev/null; then
        status=running
    elif [ -e "$session.rec" ]; then status=stopped
    else
        status=finished
    fi
    # We check c/s for running and stopped Johns.
    # Also we pick time and guesses up.
    # We unset variables to not show them for finished Johns.
    guesses=''
    time=''
    cs=''
    if [ $status = running -o $status = stopped ]; then
        # NOTE: John could print percentage. However it is not
        #       mandatory. Also it could show ETA. So we cut them out.
        gtc=$($JOHN -status:"$session" 2>&1 | sed 's/[0-9.]\+%//; s/(.*)//; s/ \+/ /g')
        guesses=$(echo $gtc | cut -d' ' -f2)
        time=$(echo $gtc | cut -d' ' -f4)
        cs=$(echo $gtc | cut -d' ' -f6)
    fi
    if [ -z "$guesses" ]; then
        guesses=0
    fi
    if [ -z "$time" ]; then
        time=0:0:0:0
    fi
    if [ -z "$cs" ]; then
        cs=0
    fi
    # We rip log.
    # We assume that time John runs is time on the last line.
    ltime=$(tail -n 1 "$session.log" | while read ltime unused_tail; do
        echo $ltime
    done)
    if [ -z "$ltime" ]; then
        ltime=0:0:0:0
    fi
    # We pick count of guesses.
    lguesses=$(grep '+ Cracked' "$session.log" | wc -l)
    # We pick time of last guess.
    llasttime=$(grep '+ Cracked' "$session.log" | while read tllasttime unused_tail; do
        echo $tllasttime
    done)
    if [ -z "$llasttime" ]; then
        llasttime=0:0:0:0
    fi
    # We calculate times.
    times=''
    # We print four times to parser: time from john, last crack time,
    # time from log, last crack time again.
    set $( (echo $time; echo $llasttime; echo $ltime; echo $llasttime) | sed 's/:/ /g' | while read days hours minutes seconds; do
        echo - $(expr $seconds + $minutes '*' 60 + $hours '*' 3600 + $days '*' 3600 '*' 24)
    done)
    times="$@"
    # We split our times into 2 expressions.
    times=$(echo $times | sed 's/-/a/; s/-/b/; s/a/-/')
    time1=$(echo $times | cut -db -f1)
    time2=$(echo $times | cut -db -f2)
    # We evaluate expressions.
    # TODO: Both times could be inaccurate for some situations.
    # NOTE: time1 is expected to be more accurate.
    time1=$(expr $time1)
    time2=$(expr $time2)
    # Tries
    # We need to substitute M, K with its meanings.
    fullcs=$(echo $cs | sed 's/M/000000/; s/K/000/')
    [ $cs ] && tries=$(expr $fullcs '*' $time1)
    tries=$(echo $tries | sed 's/000000$/M/; s/000$/K/')
    # Print
    # TODO: Prettify time since last crack.
    echo "<tr><td>$type </td><td> $status </td><td> $cs </td><td>
    $guesses ($lguesses) </td><td> $time ($ltime) </td><td> $time1
    ($time2) </td><td> $tries </td></tr>"
done < typed_files.list
echo '</table></body></html>') > astatus.html
w3m -dump astatus.html
